@@ -15,12 +15,10 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
-from gi.repository import Gtk, Gdk, GdkPixbuf
-import cairo
-
-from .abstract_canvas_tool import AbstractCanvasTool
-
-from .utilities import utilities_add_unit_to_spinbtn
+from gi.repository import Gdk, GdkPixbuf
+from .abstract_transform_tool import AbstractCanvasTool
+from .optionsbar_crop import OptionsBarCrop
+from .utilities_overlay import utilities_show_handles_on_context
 
 class ToolCrop(AbstractCanvasTool):
 	__gtype_name__ = 'ToolCrop'
@@ -28,27 +26,23 @@ class ToolCrop(AbstractCanvasTool):
 	def __init__(self, window):
 		super().__init__('crop', _("Crop"), 'tool-crop-symbolic', window)
 		self.cursor_name = 'not-allowed'
-		self.apply_to_selection = False
 		self.x_press = 0
 		self.y_press = 0
-		self.move_instead_of_crop = False
+		self.unclicked = True
+		self.add_tool_action_enum('crop-expand', 'initial')
+		self._expansion_color = 0 # transparent black, will be updated later
 
-		builder = Gtk.Builder.new_from_resource( \
-		                  '/com/github/maoschanz/drawing/tools/ui/tool_crop.ui')
-		self.bottom_panel = builder.get_object('bottom-panel')
-		self.centered_box = builder.get_object('centered_box')
-		self.cancel_btn = builder.get_object('cancel_btn')
-		self.apply_btn = builder.get_object('apply_btn')
+	def try_build_pane(self):
+		self.pane_id = 'crop'
+		self.window.options_manager.try_add_bottom_pane(self.pane_id, self)
 
-		self.height_btn = builder.get_object('height_btn')
-		self.width_btn = builder.get_object('width_btn')
-		utilities_add_unit_to_spinbtn(self.height_btn, 4, 'px')
-		utilities_add_unit_to_spinbtn(self.width_btn, 4, 'px')
+	def build_bottom_pane(self):
+		bar = OptionsBarCrop()
+		self.height_btn = bar.height_btn
+		self.width_btn = bar.width_btn
 		self.width_btn.connect('value-changed', self.on_width_changed)
 		self.height_btn.connect('value-changed', self.on_height_changed)
-		# TODO X et Y ? top/bottom/left/right ?
-
-		self.window.bottom_panel_box.add(self.bottom_panel)
+		return bar
 
 	def get_edition_status(self):
 		if self.apply_to_selection:
@@ -59,7 +53,7 @@ class ToolCrop(AbstractCanvasTool):
 	############################################################################
 
 	def on_tool_selected(self, *args):
-		self.apply_to_selection = self.selection_is_active()
+		super().on_tool_selected()
 		self._x = 0
 		self._y = 0
 		if self.apply_to_selection:
@@ -68,6 +62,8 @@ class ToolCrop(AbstractCanvasTool):
 			self.init_if_main()
 		self.width_btn.set_value(self.original_width)
 		self.height_btn.set_value(self.original_height)
+		self.set_action_sensitivity('crop-expand', not self.apply_to_selection)
+		self.build_and_do_op()
 
 	def init_if_selection(self):
 		self.original_width = self.get_selection().selection_pixbuf.get_width()
@@ -83,6 +79,31 @@ class ToolCrop(AbstractCanvasTool):
 
 	############################################################################
 
+	def _update_expansion_color(self, event_btn=1):
+		"""When the canvas grows, the color of the new pixels is parametrable"""
+		color_type = self.get_option_value('crop-expand')
+		if color_type == 'initial':
+			exp_rgba = self.get_image().get_initial_rgba()
+		elif color_type == 'secondary':
+			if event_btn == 1:
+				exp_rgba = self.window.options_manager.get_right_color()
+			if event_btn == 3:
+				exp_rgba = self.window.options_manager.get_left_color()
+		else: # color_type == 'alpha':
+			exp_rgba = Gdk.RGBA(red=1.0, green=1.0, blue=1.0, alpha=0.0)
+		self._expansion_color = self._rgba_as_hexa_int(exp_rgba)
+
+	def _rgba_as_hexa_int(self, gdk_rgba):
+		"""The method GdkPixbuf.Pixbuf.fill wants an hexadecimal integer whose
+		format is 0xrrggbbaa so here are ugly binary operators."""
+		r = int(255 * gdk_rgba.red)
+		g = int(255 * gdk_rgba.green)
+		b = int(255 * gdk_rgba.blue)
+		a = int(255 * gdk_rgba.alpha)
+		return (((((r << 8) + g) << 8) + b) << 8) + a
+
+	############################################################################
+
 	def get_width(self):
 		return self.width_btn.get_value_as_int()
 
@@ -90,64 +111,12 @@ class ToolCrop(AbstractCanvasTool):
 		return self.height_btn.get_value_as_int()
 
 	def on_width_changed(self, *args):
-		self.update_temp_pixbuf()
+		if self.unclicked:
+			self.build_and_do_op()
 
 	def on_height_changed(self, *args):
-		self.update_temp_pixbuf()
-
-	def on_unclicked_motion_on_area(self, event, surface):
-		cursor_name = ''
-		if event.y < 0.3 * surface.get_height():
-			cursor_name = cursor_name + 'n'
-		elif event.y > 0.6 * surface.get_height():
-			cursor_name = cursor_name + 's'
-
-		if event.x < 0.3 * surface.get_width():
-			cursor_name = cursor_name + 'w'
-		elif event.x > 0.6 * surface.get_width():
-			cursor_name = cursor_name + 'e'
-
-		if cursor_name == '':
-			cursor_name = 'not-allowed'
-		else:
-			cursor_name = cursor_name + '-resize'
-		self.cursor_name = cursor_name
-		self.window.set_cursor(True)
-
-	def on_press_on_area(self, event, surface, tool_width, left_color, right_color, event_x, event_y):
-		self.x_press = event.x
-		self.y_press = event.y
-
-	def on_motion_on_area(self, event, surface, event_x, event_y):
-		delta_x = event.x - self.x_press
-		delta_y = event.y - self.y_press
-
-		if self.cursor_name == 'not-allowed':
-			return
-		elif self.cursor_name == 'n-resize':
-			self.move_north(delta_y)
-		elif self.cursor_name == 'ne-resize':
-			self.move_north(delta_y)
-			self.move_east(delta_x)
-		elif self.cursor_name == 'e-resize':
-			self.move_east(delta_x)
-		elif self.cursor_name == 'se-resize':
-			self.move_south(delta_y)
-			self.move_east(delta_x)
-		elif self.cursor_name == 's-resize':
-			self.move_south(delta_y)
-		elif self.cursor_name == 'sw-resize':
-			self.move_south(delta_y)
-			self.move_west(delta_x)
-		elif self.cursor_name == 'w-resize':
-			self.move_west(delta_x)
-		elif self.cursor_name == 'nw-resize':
-			self.move_north(delta_y)
-			self.move_west(delta_x)
-
-		self.x_press = event.x
-		self.y_press = event.y
-		self.update_temp_pixbuf()
+		if self.unclicked:
+			self.build_and_do_op()
 
 	def move_north(self, delta):
 		self.height_btn.set_value(self.height_btn.get_value() - delta)
@@ -163,45 +132,99 @@ class ToolCrop(AbstractCanvasTool):
 		self.width_btn.set_value(self.width_btn.get_value() - delta)
 		self._x = self._x + delta
 
+	############################################################################
+
+	def on_unclicked_motion_on_area(self, event, surface):
+		self.cursor_name = self.get_handle_cursor_name(event.x, event.y)
+		self.window.set_cursor(True)
+
+	def on_press_on_area(self, event, surface, event_x, event_y):
+		self.x_press = event_x
+		self.y_press = event_y
+		self.unclicked = False
+		self._update_expansion_color(event.button)
+
+	def on_motion_on_area(self, event, surface, event_x, event_y):
+		delta_x = event_x - self.x_press
+		delta_y = event_y - self.y_press
+
+		if self.cursor_name == 'not-allowed':
+			return
+		else:
+			directions = self.cursor_name.replace('-resize', '')
+		if 'n' in directions:
+			self.move_north(delta_y)
+		if 's' in directions:
+			self.move_south(delta_y)
+		if 'w' in directions:
+			self.move_west(delta_x)
+		if 'e' in directions:
+			self.move_east(delta_x)
+
+		if self.apply_to_selection:
+			self._x = max(0, self._x)
+			self._y = max(0, self._y)
+
+		self.x_press = event_x
+		self.y_press = event_y
+		# self.build_and_do_op() # better UX but slower to compute
+		if not self.apply_to_selection:
+			self.build_and_do_op()
+
 	def on_release_on_area(self, event, surface, event_x, event_y):
+		self.unclicked = True
+		self.build_and_do_op()
 		self.window.set_cursor(False)
 
 	############################################################################
 
+	def on_draw(self, area, cairo_context):
+		if self.apply_to_selection:
+			x1 = int(self._x)
+			y1 = int(self._y)
+		else:
+			x1 = 0
+			y1 = 0
+		x2 = x1 + self.get_width()
+		y2 = y1 + self.get_height()
+		x1, x2, y1, y2 = self.get_image().get_corrected_coords(x1, x2, y1, y2, \
+		                                         self.apply_to_selection, False)
+		utilities_show_handles_on_context(cairo_context, x1, x2, y1, y2)
+
+	############################################################################
+
 	def build_operation(self):
 		operation = {
 			'tool_id': self.id,
 			'is_selection': self.apply_to_selection,
 			'is_preview': True,
-			'x': int(self._x),
-			'y': int(self._y),
+			'local_dx': int(self._x),
+			'local_dy': int(self._y),
 			'width': self.get_width(),
-			'height': self.get_height()
+			'height': self.get_height(),
+			'rgba': self._expansion_color
 		}
 		return operation
 
 	def do_tool_operation(self, operation):
-		if operation['tool_id'] != self.id:
-			return
-		self.restore_pixbuf()
-		x = operation['x']
-		y = operation['y']
+		self.start_tool_operation(operation)
+		x = operation['local_dx']
+		y = operation['local_dy']
 		width = operation['width']
 		height = operation['height']
-		if operation['is_selection']:
+		rgba = operation['rgba']
+		is_selection = operation['is_selection']
+		if is_selection:
 			source_pixbuf = self.get_selection_pixbuf()
 		else:
 			source_pixbuf = self.get_main_pixbuf()
 		self.get_image().set_temp_pixbuf(source_pixbuf.copy())
-		self.crop_temp_pixbuf(x, y, width, height, operation['is_selection'])
+		self.crop_temp_pixbuf(x, y, width, height, is_selection, rgba)
+		self.common_end_operation(operation)
 
-		if not operation['is_selection'] and operation['is_preview']:
-			self.scale_temp_pixbuf_to_area(width, height)
-		self.common_end_operation(operation['is_preview'], operation['is_selection'])
-
-	def crop_temp_pixbuf(self, x, y, width, height, is_selection):
+	def crop_temp_pixbuf(self, x, y, width, height, is_selection, rgba):
 		new_pixbuf = GdkPixbuf.Pixbuf.new(GdkPixbuf.Colorspace.RGB, True, 8, width, height)
-		new_pixbuf.fill(0)
+		new_pixbuf.fill(rgba)
 		src_x = max(x, 0)
 		src_y = max(y, 0)
 		if is_selection:
@@ -216,24 +239,6 @@ class ToolCrop(AbstractCanvasTool):
 		temp_p.copy_area(src_x, src_y, min_w, min_h, new_pixbuf, dest_x, dest_y)
 		self.get_image().set_temp_pixbuf(new_pixbuf)
 
-	def scale_temp_pixbuf_to_area(self, width, height):
-		visible_w = self.get_image().get_widget_width()
-		visible_h = self.get_image().get_widget_height()
-		w_ratio = visible_w/width
-		h_ratio = visible_h/height
-		if w_ratio > 1.0 and h_ratio > 1.0:
-			nice_w = width
-			nice_h = height
-		elif visible_h/visible_w > height/width:
-			nice_w = visible_w
-			nice_h = int(height * w_ratio)
-		else:
-			nice_w = int(width * h_ratio)
-			nice_h = visible_h
-		pb = self.get_image().temp_pixbuf
-		self.get_image().set_temp_pixbuf( \
-		            pb.scale_simple(nice_w, nice_h, GdkPixbuf.InterpType.TILES))
-
 	############################################################################
 ################################################################################
 
