@@ -19,6 +19,8 @@
 import os
 from gi.repository import Gtk, Gdk, Gio, GdkPixbuf, GLib
 
+from .gi_composites import GtkTemplate
+
 # Import tools
 from .tool_arc import ToolArc
 from .tool_eraser import ToolEraser
@@ -54,6 +56,7 @@ from .deco_manager import DrDecoManagerMenubar, \
 from .saving_manager import DrSavingManager
 
 from .utilities import utilities_add_filechooser_filters
+from .utilities import utilities_gfile_is_image
 
 UI_PATH = '/com/github/maoschanz/drawing/ui/'
 
@@ -73,27 +76,28 @@ DEFAULT_TOOL_ID = 'pencil'
 
 ################################################################################
 
-@Gtk.Template(resource_path=UI_PATH+'window.ui')
+@GtkTemplate(ui=UI_PATH+'window.ui')
 class DrWindow(Gtk.ApplicationWindow):
 	__gtype_name__ = 'DrWindow'
 
 	_settings = Gio.Settings.new('com.github.maoschanz.drawing')
 
 	# Window empty widgets
-	tools_flowbox = Gtk.Template.Child()
-	toolbar_box = Gtk.Template.Child()
-	info_bar = Gtk.Template.Child()
-	info_label = Gtk.Template.Child()
-	info_action = Gtk.Template.Child()
-	notebook = Gtk.Template.Child()
-	bottom_panes_box = Gtk.Template.Child()
-	tools_scrollable_box = Gtk.Template.Child()
-	tools_nonscrollable_box = Gtk.Template.Child()
-	fullscreen_btn = Gtk.Template.Child()
-	fullscreen_icon = Gtk.Template.Child()
+	tools_flowbox = GtkTemplate.Child()
+	toolbar_box = GtkTemplate.Child()
+	info_bar = GtkTemplate.Child()
+	info_label = GtkTemplate.Child()
+	info_action = GtkTemplate.Child()
+	notebook = GtkTemplate.Child()
+	bottom_panes_box = GtkTemplate.Child()
+	tools_scrollable_box = GtkTemplate.Child()
+	tools_nonscrollable_box = GtkTemplate.Child()
+	fullscreen_btn = GtkTemplate.Child()
+	fullscreen_icon = GtkTemplate.Child()
 
 	def __init__(self, **kwargs):
 		super().__init__(**kwargs)
+		self.init_template()
 		self.app = kwargs['application']
 
 		self.fullscreened = False
@@ -101,6 +105,7 @@ class DrWindow(Gtk.ApplicationWindow):
 		                   # manage several tabs in a single window despite the
 		                                      # notebook widget being pure shit
 		self.active_tool_id = None
+		self._is_tools_initialisation_finished = False
 
 		if self._settings.get_boolean('maximized'):
 			self.maximize()
@@ -119,17 +124,27 @@ class DrWindow(Gtk.ApplicationWindow):
 		self.saving_manager = DrSavingManager(self)
 
 		self.add_all_win_actions()
-		if get_cb:
-			self.build_image_from_clipboard()
-		elif gfile is not None:
-			self.build_new_tab(gfile=gfile)
-		else:
-			self.build_new_image()
-		self.init_tools()
+		self._init_tools()
 		self.connect_signals()
+
+		# The picture is built as late as possible in the init process because
+		# reading files is too prone to exceptions that may fuck everything up,
+		# and i don't trust that catching them is enough to ensure the process
+		# can continue normally.
+		try:
+			if get_cb:
+				self.build_image_from_clipboard()
+			elif gfile is not None:
+				self.build_new_tab(gfile=gfile)
+			else:
+				self.build_new_image()
+		except Exception as excp:
+			self.prompt_message(True, excp.message)
+
+		self._enable_first_tool()
 		self.set_picture_title()
 
-	def init_tools(self):
+	def _init_tools(self):
 		"""Initialize all tools, building the UI for them including the menubar,
 		and enable the default tool."""
 		disabled_tools = self._settings.get_strv('disabled-tools')
@@ -159,21 +174,32 @@ class DrWindow(Gtk.ApplicationWindow):
 		self._load_tool('filters', ToolFilters, disabled_tools, dev)
 
 		# Side pane buttons for tools, and their menubar items if they don't
-		# exist yet
+		# exist yet (they're defined on the application level)
 		self._build_tool_rows()
 		if not self.app.has_tools_in_menubar:
 			self.build_menubar_tools_menu()
 
-		# Initialisation of options and menus
+		# Initialisation of which tool is active
 		tool_id = self._settings.get_string('last-active-tool')
 		if tool_id not in self.tools:
 			tool_id = DEFAULT_TOOL_ID
 		self.active_tool_id = tool_id
 		self.former_tool_id = tool_id
-		if tool_id == DEFAULT_TOOL_ID: # The "pencil" button is already active
-			self.enable_tool(tool_id)
+		# the end of this process, implemented in the `_enable_first_tool`
+		# method, will be called later because it requires an active image,
+		# which doesn't exist yet at this point of the window init process.
+
+	def _enable_first_tool(self):
+		"""Near the end of the window initialisation process, this method is
+		called once to make sure all things related to the active tool, mostly
+		its options' values, and the visibility of its optionsbar, are all
+		correctly loaded."""
+		if self.active_tool_id == DEFAULT_TOOL_ID:
+			# Special case of the "pencil" radio button, which is already active
+			self.enable_tool(self.active_tool_id)
 		else:
 			self.active_tool().row.set_active(True)
+		self._is_tools_initialisation_finished = True
 
 	def _load_tool(self, tool_id, tool_class, disabled_tools, dev):
 		"""Given its id and its python class, this method tries to load a tool,
@@ -202,14 +228,24 @@ class DrWindow(Gtk.ApplicationWindow):
 
 	def build_menubar_tools_menu(self):
 		sections = [None, None, None]
-		sections[2] = self.get_menubar_item([[True, 4], [False, 0]])
-		sections[0] = self.get_menubar_item([[True, 4], [False, 1]])
-		sections[1] = self.get_menubar_item([[True, 4], [False, 2]])
+		# the ids of the various kinds of tools don't match the index of their
+		# section in the submenu
+		sections[2] = self._get_menubar_tools_section(0)
+		sections[0] = self._get_menubar_tools_section(1)
+		sections[1] = self._get_menubar_tools_section(2)
 		for tool_id in self.tools:
 			tool = self.tools[tool_id]
 			tool.add_item_to_menu(sections[tool.menu_id])
 		self.app.has_tools_in_menubar = True
 
+	def _get_menubar_tools_section(self, section_index):
+		return self._get_menubar_item([
+			[True, 4],
+			[False, 1],
+			[True, 0],
+			[False, section_index]
+		])
+
 	############################################################################
 	# TABS AND WINDOWS MANAGEMENT ##############################################
 
@@ -220,8 +256,7 @@ class DrWindow(Gtk.ApplicationWindow):
 		height = self._settings.get_int('default-height')
 		rgba = self._settings.get_strv('default-rgba')
 		self.build_new_tab(width=width, height=height, background_rgba=rgba)
-		if self.active_tool_id is not None: # Tools might not be initialized yet
-			self.set_picture_title()
+		self.set_picture_title()
 
 	def build_new_custom(self, *args):
 		"""Open a new tab with a drawable blank image using the custom values
@@ -239,7 +274,11 @@ class DrWindow(Gtk.ApplicationWindow):
 		empty, the new image will be blank."""
 		cb = Gtk.Clipboard.get(Gdk.SELECTION_CLIPBOARD)
 		pixbuf = cb.wait_for_image()
-		self.build_new_tab(pixbuf=pixbuf)
+		if pixbuf is None:
+			self.prompt_message(True, _("The clipboard doesn't contain any image."))
+			self.build_new_image()
+		else:
+			self.build_new_tab(pixbuf=pixbuf)
 
 	def build_image_from_selection(self, *args):
 		"""Open a new tab with the image in the selection."""
@@ -256,12 +295,17 @@ class DrWindow(Gtk.ApplicationWindow):
 			new_image.try_load_file(gfile)
 		elif pixbuf is not None:
 			new_image.try_load_pixbuf(pixbuf)
+			# XXX dans l'idéal on devrait ne rien ouvrir non ? ou si besoin (si
+			# ya pas de fenêtre) ouvrir un truc respectant les settings, plutôt
+			# qu'un petit pixbuf rouge
 		else:
 			new_image.init_background(width, height, background_rgba)
 		self.update_tabs_visibility()
 		self.notebook.set_current_page(self.notebook.get_n_pages()-1)
 
 	def on_active_tab_changed(self, *args):
+		if not self._is_tools_initialisation_finished:
+			return
 		self.switch_to(self.active_tool_id, args[1])
 		# print("changement d'image")
 		self.set_picture_title(args[1].update_title())
@@ -271,7 +315,7 @@ class DrWindow(Gtk.ApplicationWindow):
 
 	def update_tabs_menu_section(self, *args):
 		action = self.lookup_action('active_tab')
-		section = self.get_menubar_item([[True, 2], [False, 1]])
+		section = self._get_menubar_item([[True, 2], [False, 1]])
 		section.remove_all()
 		for page in self.notebook.get_children():
 			tab_title = page.update_title()
@@ -360,7 +404,7 @@ class DrWindow(Gtk.ApplicationWindow):
 		self.notebook.drag_dest_add_uri_targets()
 		# because drag_dest_add_image_targets doesn't work for files
 
-	def get_menubar_item(self, path_description):
+	def _get_menubar_item(self, path_description):
 		"""Get an item of the app-wide menubar. The `path_description` object
 		is an array of [boolean, int] couples. The boolean means if we're
 		looking for a submenu, the int is an index."""
@@ -411,13 +455,15 @@ class DrWindow(Gtk.ApplicationWindow):
 
 		self.add_action_boolean('toggle_preview', False, self.action_toggle_preview)
 		self.app.set_accels_for_action('win.toggle_preview', ['<Ctrl>m'])
-		self.add_action_boolean('show_labels', self._settings.get_boolean( \
-		                     'show-labels'), self.on_show_labels_action_changed)
+
+		show_labels = self._settings.get_boolean('show-labels')
+		self.add_action_boolean('show_labels', show_labels, self.action_show_labels)
 		self.app.set_accels_for_action('win.show_labels', ['F9'])
 
+		self.add_action_simple('fullscreen', self.action_fullscreen, ['F11'])
+
 		self.add_action_simple('reload_file', self.action_reload, ['<Ctrl>r'])
 		self.add_action_simple('properties', self.action_properties, None)
-		self.add_action_simple('fullscreen', self.action_fullscreen, ['F11'])
 		self.add_action_simple('unfullscreen', self.action_unfullscreen, ['Escape'])
 
 		self.add_action_simple('go_up', self.action_go_up, ['<Ctrl>Up'])
@@ -449,7 +495,7 @@ class DrWindow(Gtk.ApplicationWindow):
 		self.add_action_simple('save_alphaless', self.action_save_alphaless, None)
 		self.add_action_simple('save_as', self.action_save_as, ['<Ctrl><Shift>s'])
 		self.add_action_simple('export_as', self.action_export_as, None)
-		self.add_action_simple('to_clipboard', self.action_export_cb, None)
+		self.add_action_simple('to_clipboard', self.action_export_cb, ['<Ctrl><Shift>c'])
 		self.add_action_simple('print', self.action_print, None)
 
 		self.add_action_simple('import', self.action_import, ['<Ctrl>i'])
@@ -463,7 +509,7 @@ class DrWindow(Gtk.ApplicationWindow):
 
 		self.add_action_simple('back_to_previous', self.back_to_previous, ['<Ctrl>b'])
 		self.add_action_simple('force_selection', self.force_selection, None)
-		self.add_action_simple('apply_transform_tool', self.action_apply_transformation, None)
+		self.add_action_simple('apply_transform', self.action_apply_transform, ['<Ctrl>Return'])
 
 		self.add_action_enum('active_tool', DEFAULT_TOOL_ID, self.on_change_active_tool)
 
@@ -587,15 +633,15 @@ class DrWindow(Gtk.ApplicationWindow):
 		builder = Gtk.Builder.new_from_resource(UI_PATH + 'win-menus.ui')
 		fullscreen_menu = builder.get_object('fullscreen-menu')
 
-		tabs_list = self.get_menubar_item([[True, 2], [False, 1]])
+		tabs_list = self._get_menubar_item([[True, 2], [False, 1]])
 		fullscreen_menu.append_section(_("Opened images"), tabs_list)
 
-		classic_tools_section = self.get_menubar_item([[True, 4], [False, 1]])
+		classic_tools_section = self._get_menubar_tools_section(1)
 		section = fullscreen_menu.get_item_link(3, Gio.MENU_LINK_SECTION)
 		section.prepend_section(None, classic_tools_section)
 
-		selection_tools_section = self.get_menubar_item([[True, 4], [False, 0]])
-		transform_tools_section = self.get_menubar_item([[True, 4], [False, 2]])
+		selection_tools_section = self._get_menubar_tools_section(0)
+		transform_tools_section = self._get_menubar_tools_section(2)
 		submenu = section.get_item_link(1, Gio.MENU_LINK_SUBMENU)
 		submenu.append_section(None, selection_tools_section)
 		submenu.append_section(None, transform_tools_section)
@@ -626,6 +672,8 @@ class DrWindow(Gtk.ApplicationWindow):
 		self._decorations.adapt_to_window_size()
 
 		available_width = self.bottom_panes_box.get_allocated_width()
+		if not self._is_tools_initialisation_finished:
+			return # there is no active pane nor active image yet
 		self.options_manager.adapt_to_window_size(available_width)
 
 		self.get_active_image().fake_scrollbar_update()
@@ -639,7 +687,7 @@ class DrWindow(Gtk.ApplicationWindow):
 		self.info_action.set_visible(False)
 		if show:
 			self.info_label.set_label(label)
-		if self._settings.get_boolean('devel-only'):
+		if show or self._settings.get_boolean('devel-only'):
 			print('Drawing: ' + label)
 
 	def prompt_action(self, message, action_name, action_label):
@@ -658,6 +706,7 @@ class DrWindow(Gtk.ApplicationWindow):
 	# FULLSCREEN ###############################################################
 
 	def action_unfullscreen(self, *args):
+		"""Simple action, exiting fullscreen."""
 		# TODO connect to signals instead
 		self.unfullscreen()
 		self.set_fullscreen_state(False)
@@ -706,7 +755,7 @@ class DrWindow(Gtk.ApplicationWindow):
 		# TODO https://lazka.github.io/pgi-docs/Gio-2.0/classes/Settings.html#Gio.Settings.create_action
 		self.set_tools_labels_visibility(self._settings.get_boolean('show-labels'))
 
-	def on_show_labels_action_changed(self, *args):
+	def action_show_labels(self, *args):
 		show_labels = not args[0].get_state()
 		self._settings.set_boolean('show-labels', show_labels)
 		args[0].set_state(GLib.Variant.new_boolean(show_labels))
@@ -900,9 +949,25 @@ class DrWindow(Gtk.ApplicationWindow):
 		cancel_id = dialog.set_action(_("Cancel"), None, False)
 		open_id = dialog.set_action(_("Open"), None, False)
 		import_id = dialog.set_action(_("Import"), None, True)
+
 		uris = data.get_uris()
-		if len(uris) == 1:
-			label = uris[0].split('/')[-1]
+		gfiles = []
+		for uri in uris:
+			try:
+				gfile = Gio.File.new_for_uri(uri)
+				is_image, error_msg = utilities_gfile_is_image(gfile)
+			except Exception as excp:
+				is_image = False
+				error_msg = excp.message
+			if is_image:
+				gfiles.append(gfile)
+			else:
+				self.prompt_message(True, error_msg)
+
+		if len(gfiles) == 0:
+			return
+		elif len(gfiles) == 1:
+			label = gfiles[0].get_path().split('/')[-1]
 		else:
 			# Context for translation:
 			# "What do you want to do with *these files*?"
@@ -912,16 +977,12 @@ class DrWindow(Gtk.ApplicationWindow):
 		dialog.add_string(_("What do you want to do with %s?") % label)
 		result = dialog.run()
 		dialog.destroy()
-		for uri in uris:
-			# print(uri)
-			# valider l'URI TODO
-			if result == import_id:
-				f = Gio.File.new_for_uri(uri)
-				self.import_from_path(f.get_path())
-				return
-			elif result == open_id:
-				f = Gio.File.new_for_uri(uri)
+
+		if result == open_id:
+			for f in gfiles:
 				self.build_new_tab(gfile=f)
+		elif result == import_id:
+			self.import_from_path(gfiles[0].get_path())
 
 	def try_load_file(self, gfile):
 		if gfile is not None:
@@ -1036,7 +1097,7 @@ class DrWindow(Gtk.ApplicationWindow):
 	def force_selection(self, *args):
 		self.get_selection_tool().row.set_active(True) # XXX not enable_tool?
 
-	def action_apply_transformation(self, *args):
+	def action_apply_transform(self, *args):
 		self.active_tool().on_apply_temp_pixbuf_tool_operation()
 
 	############################################################################
